/*
   voiceGUI.c
   code for listing and selecting Gnaural voices 
   Depends on:
   glib.h
   gtk/gtk.h
   main.h
   voiceGUI.h
   ScheduleGUI.h

   Theory: give main_UpdateGUI_Voices() a GtkVBox, and this fills it with a table

   Copyright (C) 2008  Bret Logan

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib.h>
#include <gtk/gtk.h>
#include "main.h"
#include "voiceGUI.h"
#include "ScheduleGUI.h"

/////////////////////////////////////////////////////
//This sets the column ordering for the overall view
enum
{
 VOICEGUI_VIEW_COLUMN,
 VOICEGUI_MUTE_COLUMN,
 VOICEGUI_TYPE_COLUMNS,
 VOICEGUI_DESC_COLUMN,
 VOICEGUI_COUNTTOT_COLUMNS,
 VOICEGUI_COUNTSEL_COLUMNS,
 VOICEGUI_END_COLUMNS
};

/////////////////////////////////////////////////////
//20070629: returns an SG_Voice for a given index value,
//or NULL if there is none.
SG_Voice *main_VoiceGetIndex (int index)
{
 int voiceindex = 0;
 SG_Voice *curVoice = SG_FirstVoice;

 while (curVoice != NULL)
 {
  if (voiceindex == index)
  {
   break;
  }
  //main loop update
  ++voiceindex;
  curVoice = curVoice->NextVoice;
 }
 return curVoice;
}

/////////////////////////////////////////////////////
void main_callback_VoicesMute (GtkWidget * widget, gpointer data)
{
 //g_print ("Checkbox A%d toggled\n", (int) data);
 int voice = (int) data;

 SG_Voice *curVoice = main_VoiceGetIndex (voice);

 if (curVoice == NULL)
 {
  return;
 }

 if (voice >= BB_VoiceCount)
 {
  // g_print ("voice: %d, BB_VoiceCount: %d\n", (int) data, BB_VoiceCount);
  return;
 }
 if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
 {
  BB_Voice[voice].mute = curVoice->mute = TRUE;
 }
 else
 {
  BB_Voice[voice].mute = curVoice->mute = FALSE;
 }
}

/////////////////////////////////////////////////////
//handles the view/hide checkbox in the Voices Box
void main_callback_VoicesView (GtkWidget * widget, gpointer data)
{
 //g_print ("Checkbox B%d toggled\n", (int) data);
 SG_Voice *curVoice = main_VoiceGetIndex ((int) data);

 if (curVoice == NULL)
 {
  return;
 }
 if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
 {
  curVoice->hide = FALSE;
 }
 else
 {
  curVoice->hide = TRUE;
  SG_VoiceTestLegalSelection ();
  //SG_SelectDataPoints_Voice (curVoice, FALSE);//it really might be better to call this
  if (curVoice->state == SG_SELECTED)
  {
   curVoice->state = SG_UNSELECTED;
  }
 }
}

/////////////////////////////////////////////////////
//20070629: handles selecting voices and launching
//properties box:
gboolean main_callback_VoicesProperties (GtkWidget * widget,
                                         GdkEventButton * event,
                                         gpointer data)
{
 // g_print ("Voice Label %d clicked, mouse button %d\n", (int) data, event->button);
 SG_Voice *curVoice = main_VoiceGetIndex ((int) data);

 if (curVoice != NULL)
 {
  SG_SelectVoice (curVoice);
  if (event->button == 3)
  {
   main_VoicePropertiesDialog (main_drawing_area, curVoice);
  }
  main_UpdateGUI_Voices (main_vboxVoices);
 }
 return FALSE;
}

/////////////////////////////////////////////////////
//a20070629 box is simply where you want list put
void main_UpdateGUI_Voices (GtkVBox * mainbox)
{
 GtkWidget *CheckBut;
 GtkWidget *labelVoice;
 GtkWidget *eventbox;
 int voiceindex = 0;
 static GtkWidget *box = NULL;  //create so I can delete something inside box user passed me

 //first create something other than user's box that can be deleted
 if (box != NULL)
 {
  gtk_widget_destroy (box);
 }
 box = gtk_vbox_new (FALSE, 0);
 gtk_widget_show (box);
 gtk_box_pack_start (GTK_BOX (mainbox), box, FALSE, FALSE, 0);

 //create the new hbox for this row:
 GtkWidget *boxh = gtk_hbox_new (FALSE, 0);

 gtk_widget_show (boxh);

 //give a heading for the list:
 //do some chores to get some numbers:
 int dpcount = 0;
 int seldpcount = 0;
 int voicecount = 0;
 int selvoicecount = 0;

 SG_CountAllData (&voicecount, &selvoicecount, &dpcount, &seldpcount);
 SG_DBGOUT ("Updating Voice View");
 sprintf (main_sTmp, "View/Mute/Type/Description/Counts (%d,%d) (%d/%d)",
          dpcount, seldpcount, selvoicecount, voicecount);
 labelVoice = gtk_label_new (main_sTmp);
 gtk_box_pack_start (GTK_BOX (boxh), labelVoice, FALSE, FALSE, 0);
 gtk_widget_show (labelVoice);
 gtk_box_pack_start (GTK_BOX (box), boxh, FALSE, FALSE, 0);

 //main loop:
 voiceindex = 0;
 SG_Voice *curVoice = SG_FirstVoice;

 while (curVoice != NULL)
 {
  //create the new hbox for this row:
  boxh = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (boxh);

  //make first checkbutton for Show/Hide:
  CheckBut = (GtkWidget *) gtk_check_button_new ();
  gtk_widget_show (CheckBut);

  //Connect the "toggled" signal of the checkbox to our callback
  g_signal_connect (G_OBJECT (CheckBut), "toggled",
                    G_CALLBACK (main_callback_VoicesView),
                    (gpointer) voiceindex);

  //add checkbox to this row:
  gtk_box_pack_start (GTK_BOX (boxh), CheckBut, FALSE, FALSE, 0);

  //check if visible:
  if (curVoice->hide == FALSE)
  {
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (CheckBut), TRUE);
  }

  //make second checkbutton for Audible/Mute:
  CheckBut = (GtkWidget *) gtk_check_button_new ();
  gtk_widget_show (CheckBut);

  // Connect the "toggled" signal of the checkbox to our callback
  g_signal_connect (G_OBJECT (CheckBut), "toggled",
                    G_CALLBACK (main_callback_VoicesMute),
                    (gpointer) voiceindex);

  //add checkbox to this row:
  gtk_box_pack_start (GTK_BOX (boxh), CheckBut, FALSE, FALSE, 0);

  //check if muted:
  if (BB_Voice[voiceindex].mute == TRUE)
  {
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (CheckBut), TRUE);
  }

  //create an eventbox to recieve mouse events:
  eventbox = gtk_event_box_new ();
  gtk_box_pack_start (GTK_BOX (boxh), eventbox, FALSE, FALSE, 0);
  gtk_widget_show (eventbox);
  // And bind an action to it:
  gtk_widget_set_events (eventbox, GDK_BUTTON_PRESS_MASK);
  g_signal_connect (G_OBJECT (eventbox), "button_press_event",
                    G_CALLBACK (main_callback_VoicesProperties),
                    (gpointer) voiceindex);

  //add label with description to put in the event box:
  main_VoiceInfoFormatter (curVoice);
  labelVoice = gtk_label_new (main_sTmp);
  // gtk_label_set_selectable ((GtkLabel *) labelVoice, TRUE);
  if (curVoice->state == SG_SELECTED)
  {
   //#define MARKUP_STRING "<span size='small'>%s</span>"
#define MARKUP_STRING "<u>%s</u>"
   gchar *markup = g_markup_printf_escaped (MARKUP_STRING, main_sTmp);

   gtk_label_set_markup (GTK_LABEL (labelVoice), markup);
   g_free (markup);
  }
  gtk_container_add (GTK_CONTAINER (eventbox), labelVoice);
  gtk_widget_show (labelVoice);

  //  sprintf (tmpstr, "%d) This is checkbox label", voiceindex);

  //finally, add the hbox and label to the vbox:
  gtk_box_pack_start (GTK_BOX (box), boxh, FALSE, FALSE, 0);

  //main loop update
  ++voiceindex;
  curVoice = curVoice->NextVoice;
 }
}
